@@ -2,9 +2,7 @@
 //
 // The Core Audio Engine component of Rivendell
 //
-//   (C) Copyright 2002-2004 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: cae.cpp,v 1.115.4.1 2012/08/03 16:52:38 cvs Exp $
+//   (C) Copyright 2002-2015 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -59,6 +57,23 @@ extern jack_client_t *jack_client;
 
 #define PRINT_COMMANDS
 
+#ifndef HAVE_SRC_CONV
+void src_int_to_float_array (const int *in, float *out, int len)
+{
+  for(int i=0;i<len;i++) {
+    out[i]=(double)in[i]/2147483648.0;
+  }
+}
+
+
+void src_float_to_int_array (const float *in, int *out, int len)
+{
+  for(int i=0;i<len;i++) {
+    out[i]=(int)(in[i]*2147483648.0);
+  }
+}
+#endif  // HAVE_SRC_CONV
+
 void LogLine(RDConfig::LogPriority prio,const QString &line)
 {
   FILE *file;
@@ -377,7 +392,7 @@ void MainObject::newConnection(int fd)
     close(fd);
     return;
   }
-  socket[i]=new RDSocket(i,this,"socket_0");
+  socket[i]=new RDSocket(i,this);
   socket[i]->setSocket(fd);
   connect(socket[i],SIGNAL(readyReadID(int)),this,SLOT(socketData(int)));
   connect(socket[i],SIGNAL(connectionClosedID(int)),
@@ -490,6 +505,11 @@ void MainObject::updateMeters()
 	  }
 	  hpiGetOutputPosition(i,positions);
 	  SendMeterPositionUpdate(i,positions);
+	  for(int j=0;j<RD_MAX_STREAMS;j++) {
+	    if(hpiGetStreamOutputMeters(i,j,levels)) {
+	      SendStreamMeterLevelUpdate(i,j,levels);
+	    }      
+	  }
 	  break;
 
 	case RDStation::Jack:
@@ -508,6 +528,11 @@ void MainObject::updateMeters()
 	  }
 	  jackGetOutputPosition(i,positions);
 	  SendMeterPositionUpdate(i,positions);
+	  for(int j=0;j<RD_MAX_STREAMS;j++) {
+	    if(jackGetStreamOutputMeters(i,j,levels)) {
+	      SendStreamMeterLevelUpdate(i,j,levels);
+	    }      
+	  }
 	  break;
 
 	case RDStation::Alsa:
@@ -526,6 +551,11 @@ void MainObject::updateMeters()
 	  }
 	  alsaGetOutputPosition(i,positions);
 	  SendMeterPositionUpdate(i,positions);
+	  for(int j=0;j<RD_MAX_STREAMS;j++) {
+	    if(alsaGetStreamOutputMeters(i,j,levels)) {
+	      SendStreamMeterLevelUpdate(i,j,levels);
+	    }      
+	  }
 	  break;
 
 	case RDStation::None:
@@ -1953,8 +1983,6 @@ int MainObject::GetHandle(int card,int stream)
 
 void MainObject::ProbeCaps(RDStation *station)
 {
-  void *handle;
-
   //
   // Patent-clear codecs
   //
@@ -1978,6 +2006,11 @@ void MainObject::ProbeCaps(RDStation *station)
   station->setHaveCapability(RDStation::HaveTwoLame,LoadTwoLame());
   station->setHaveCapability(RDStation::HaveMpg321,LoadMad());
 
+  //
+  // MP4 Decoder
+  //
+  station->setHaveCapability(RDStation::HaveMp4Decode,CheckMp4Decode());
+
 #ifdef HPI
   station->setDriverVersion(RDStation::Hpi,hpiVersion());
 #else
@@ -2034,6 +2067,17 @@ bool MainObject::CheckLame()
 }
 
 
+bool MainObject::CheckMp4Decode()
+{
+#ifdef HAVE_MP4_LIBS
+  return (dlopen("libfaad.so",RTLD_LAZY)!=NULL)&&
+    (dlopen("libmp4v2.so",RTLD_LAZY)!=NULL);
+#else
+  return false;
+#endif  // HAVE_MP4_LIBS
+}
+
+
 bool MainObject::LoadTwoLame()
 {
 #ifdef HAVE_TWOLAME
@@ -2197,6 +2241,16 @@ void MainObject::SendMeterLevelUpdate(const QString &type,int cardnum,
 }
 
 
+void MainObject::SendStreamMeterLevelUpdate(int cardnum,int streamnum,
+					    short levels[])
+{
+  char msg[1500];
+  int n=snprintf(msg,1500,"MO %d %d %d %d",
+		 cardnum,streamnum,levels[0],levels[1]);
+  SendMeterUpdate(msg,n);
+}
+
+
 void MainObject::SendMeterPositionUpdate(int cardnum,unsigned pos[])
 {
   char msg[1500];
