@@ -2,9 +2,7 @@
 //
 // Add a Rivendell Cart
 //
-//   (C) Copyright 2002-2004 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: rdadd_cart.cpp,v 1.5.2.1 2009/03/24 12:11:18 cvs Exp $
+//   (C) Copyright 2002-2004,2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -43,11 +41,13 @@
 #include <rdescape_string.h>
 
 RDAddCart::RDAddCart(QString *group,RDCart::Type *type,QString *title,
-		     const QString &username,QWidget *parent,const char *name)
-  : QDialog(parent,name,true)
+		     const QString &username,RDSystem *system,
+		     QWidget *parent)
+  : QDialog(parent,"",true)
 {
   QString sql;
   RDSqlQuery *q;
+  cart_system=system;
   cart_group=group;
   cart_type=type;
   cart_title=title;
@@ -68,11 +68,6 @@ RDAddCart::RDAddCart(QString *group,RDCart::Type *type,QString *title,
   QFont label_font("Helvetica",12,QFont::Bold);
   label_font.setPixelSize(12);
 
-  //
-  // Text Validator
-  //
-  RDTextValidator *text_validator=new RDTextValidator(this,"validator");
-
   //
   // Group
   //
@@ -149,7 +144,7 @@ RDAddCart::RDAddCart(QString *group,RDCart::Type *type,QString *title,
   cart_title_edit=new QLineEdit(this,"cart_title_edit");
   cart_title_edit->setGeometry(145,73,sizeHint().width()-155,19);
   cart_title_edit->setMaxLength(255);
-  cart_title_edit->setValidator(text_validator);
+  //  cart_title_edit->setValidator(text_validator);
   cart_title_edit->setText(tr("[new cart]"));
   QLabel *cart_title_label=
     new QLabel(cart_title_edit,tr("&New Cart Title:"),this,
