@@ -2,7 +2,7 @@
 //
 // Abstract a Rivendell Cart.
 //
-//   (C) Copyright 2002-2004 Fred Gleason <fredg@paravelsystems.com>
+//   (C) Copyright 2002-2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -861,11 +861,25 @@ void RDCart::getMetadata(RDWaveData *data) const
   QString sql;
   RDSqlQuery *q;
 
-  sql=QString("select TITLE,ARTIST,ALBUM,YEAR,LABEL,CLIENT,")+
-    "AGENCY,PUBLISHER,COMPOSER,USER_DEFINED,CONDUCTOR,SONG_ID,BPM,USAGE_CODE"+
+  sql=QString("select ")+
+    "TITLE,"+         // 00
+    "ARTIST,"+        // 01
+    "ALBUM,"+         // 02
+    "YEAR,"+          // 03
+    "LABEL,"+         // 04
+    "CLIENT,"+        // 05
+    "AGENCY,"+        // 06
+    "PUBLISHER,"+     // 07
+    "COMPOSER,"+      // 08
+    "USER_DEFINED,"+  // 09
+    "CONDUCTOR,"+     // 10
+    "SONG_ID,"+       // 11
+    "BPM,"+           // 12
+    "USAGE_CODE "+    // 13
     QString().sprintf(" from CART where NUMBER=%u",cart_number);
   q=new RDSqlQuery(sql);
   if(q->first()) {
+    data->setCartNumber(cart_number);
     data->setTitle(q->value(0).toString());
     data->setArtist(q->value(1).toString());
     data->setAlbum(q->value(2).toString());
@@ -879,7 +893,7 @@ void RDCart::getMetadata(RDWaveData *data) const
     data->setConductor(q->value(10).toString());
     data->setTmciSongId(q->value(11).toString());
     data->setBeatsPerMinute(q->value(12).toUInt());
-    data->setUsageCode(q->value(13).toUInt());
+    data->setUsageCode((RDWaveData::UsageCode)q->value(13).toUInt());
     data->setSchedCodes(schedCodesList());
     data->setMetadataFound(true);
   }
@@ -971,7 +985,7 @@ bool RDCart::validateLengths(int len) const
 }
 
 
-QString RDCart::xml(bool include_cuts) const
+QString RDCart::xml(bool include_cuts,RDSettings *settings,int cutnum) const
 {
 #ifdef WIN32
   return QString();
@@ -983,6 +997,35 @@ QString RDCart::xml(bool include_cuts) const
   RDCut *cut;
   QStringList mlist;
 
+  sql=QString("select ")+
+    "TYPE,"+                  // 00
+    "GROUP_NAME,"+            // 01
+    "TITLE,"+                 // 02
+    "ARTIST,"+                // 03
+    "ALBUM,"+                 // 04
+    "YEAR,"+                  // 05
+    "LABEL,"+                 // 06
+    "CLIENT,"+                // 07
+    "AGENCY,"+                // 08
+    "PUBLISHER,"+             // 09
+    "COMPOSER,"+              // 10
+    "USER_DEFINED,"+          // 11
+    "USAGE_CODE,"+            // 12
+    "FORCED_LENGTH,"+         // 13
+    "AVERAGE_LENGTH,"+        // 14
+    "LENGTH_DEVIATION,"+      // 15
+    "AVERAGE_SEGUE_LENGTH,"+  // 16
+    "AVERAGE_HOOK_LENGTH,"+   // 17
+    "CUT_QUANTITY,"+          // 18
+    "LAST_CUT_PLAYED,"+       // 19
+    "VALIDITY,"+              // 20
+    "ENFORCE_LENGTH,"+        // 21
+    "ASYNCRONOUS,"+           // 22
+    "OWNER,"+                 // 23
+    "METADATA_DATETIME,"+     // 24
+    "CONDUCTOR "+             // 25
+    QString().sprintf("from CART where NUMBER=%u",cart_number);
+  /*
   sql=QString().sprintf("select TYPE,GROUP_NAME,TITLE,ARTIST,ALBUM,YEAR,\
                          LABEL,CLIENT,AGENCY,PUBLISHER,COMPOSER,USER_DEFINED,\
                          USAGE_CODE,FORCED_LENGTH,AVERAGE_LENGTH,\
@@ -991,6 +1034,7 @@ QString RDCart::xml(bool include_cuts) const
                          VALIDITY,\
                          ENFORCE_LENGTH,ASYNCRONOUS,OWNER,METADATA_DATETIME \
                          from CART where NUMBER=%u",cart_number);
+  */
   q=new RDSqlQuery(sql);
   if(q->first()) {
     ret+="<cart>\n";
@@ -1017,6 +1061,7 @@ QString RDCart::xml(bool include_cuts) const
     ret+="  "+RDXmlField("agency",q->value(8).toString());
     ret+="  "+RDXmlField("publisher",q->value(9).toString());
     ret+="  "+RDXmlField("composer",q->value(10).toString());
+    ret+="  "+RDXmlField("conductor",q->value(25).toString());
     ret+="  "+RDXmlField("userDefined",q->value(11).toString());
     ret+="  "+RDXmlField("usageCode",q->value(12).toInt());
     ret+="  "+RDXmlField("forcedLength",
@@ -1025,13 +1070,12 @@ QString RDCart::xml(bool include_cuts) const
 			 RDGetTimeLength(q->value(14).toUInt(),true));
     ret+="  "+RDXmlField("lengthDeviation",
 			 RDGetTimeLength(q->value(15).toUInt(),true));
-    ret+="  "+RDXmlField("averageSegueLenth",
+    ret+="  "+RDXmlField("averageSegueLength",
 			 RDGetTimeLength(q->value(16).toUInt(),true));
     ret+="  "+RDXmlField("averageHookLength",
 			 RDGetTimeLength(q->value(17).toUInt(),true));
     ret+="  "+RDXmlField("cutQuantity",q->value(18).toUInt());
     ret+="  "+RDXmlField("lastCutPlayed",q->value(19).toUInt());
-    ret+="  "+RDXmlField("validity",q->value(20).toUInt());
     ret+="  "+RDXmlField("enforceLength",RDBool(q->value(21).toString()));
     ret+="  "+RDXmlField("asyncronous",RDBool(q->value(22).toString()));
     ret+="  "+RDXmlField("owner",q->value(23).toString());
@@ -1040,15 +1084,24 @@ QString RDCart::xml(bool include_cuts) const
       case RDCart::Audio:
 	if(include_cuts) {
 	  ret+="<cutList>\n";
-	  sql=QString().sprintf("select CUT_NAME from CUTS where CART_NUMBER=%u",
-				cart_number);
-	  q1=new RDSqlQuery(sql);
-	  while(q1->next()) {
-	    cut=new RDCut(q1->value(0).toString());
-	    ret+=cut->xml();
+	  if(cutnum<0) {
+	    sql=QString("select CUT_NAME from CUTS where ")+
+	      QString().sprintf("(CART_NUMBER=%u)",cart_number);
+	    q1=new RDSqlQuery(sql);
+	    while(q1->next()) {
+	      cut=new RDCut(q1->value(0).toString());
+	      ret+=cut->xml(settings);
+	      delete cut;
+	    }
+	    delete q1;
+	  }
+	  else {
+	    cut=new RDCut(RDCut::cutName(cart_number,cutnum));
+	    if(cut->exists()) {
+	      ret+=cut->xml(settings);
+	    }
 	    delete cut;
 	  }
-	  delete q1;
 	  ret+="</cutList>\n";
 	}
 	break;
@@ -1206,15 +1259,15 @@ void RDCart::updateLength(bool enforce_length,unsigned length)
     sql+="START_DATETIME=NULL,";
   }
   else {
-    sql+=QString().sprintf("START_DATETIME=\"%s\",",
-		(const char *)start_datetime.toString("yyyy-MM-dd hh:mm:ss"));
+    sql+=QString().sprintf("START_DATETIME=%s,",
+		(const char *)RDCheckDateTime(start_datetime,"yyyy-MM-dd hh:mm:ss"));
   }
   if(end_datetime.isNull()||(!dates_valid)) {
     sql+="END_DATETIME=NULL,";
   }
   else {
-    sql+=QString().sprintf("END_DATETIME=\"%s\",",
-		(const char *)end_datetime.toString("yyyy-MM-dd hh:mm:ss"));
+    sql+=QString().sprintf("END_DATETIME=%s,",
+		(const char *)RDCheckDateTime(end_datetime,"yyyy-MM-dd hh:mm:ss"));
   }
   sql+=QString().sprintf("VALIDITY=%u where NUMBER=%u",
 			 cart_validity,cart_number);
@@ -1475,10 +1528,16 @@ bool RDCart::removeCutAudio(RDStation *station,RDUser *user,unsigned cart_num,
   long response_code=0;
   char url[1024];
   QString xml="";
+  QString sql;
+  RDSqlQuery *q;
 
   if(user==NULL) { 
     unlink(RDCut::pathName(cutname));
     unlink(RDCut::pathName(cutname)+".energy");
+    sql=QString("delete from CUT_EVENTS where ")+
+      "CUT_NAME=\""+cutname+"\"";
+    q=new RDSqlQuery(sql);
+    delete q;
   }
   else {
     //
@@ -1534,6 +1593,334 @@ void RDCart::removePending(RDStation *station,RDUser *user,RDConfig *config)
 }
 
 
+unsigned RDCart::readXml(std::vector<RDWaveData> *data,const QString &xml)
+{
+  //
+  // HACK! HACK! HACK! HACK! HACK!
+  //
+  // This is totally ad-hoc and will likely break horribly for XML that
+  // deviates the least bit from that generated by Rivendell's own xml()
+  // methods.
+  //
+  // Slated to be put out of its misery as soon as we get a Real XML Parser.
+  //
+  int istate=0;
+  RDWaveData cartdata;
+  RDSettings *settings=NULL;
+  QStringList f0=f0.split("\n",xml);
+
+  for(unsigned i=0;i<f0.size();i++) {
+    f0[i]=f0[i].stripWhiteSpace();
+  }
+
+  for(unsigned i=0;i<f0.size();i++) {
+    switch(istate) {
+    case 0:
+      if(f0[i]=="<cart>") {
+	istate=1;
+      }
+      break;
+
+    case 1:  // Cart-level objects
+      if(f0[i].contains("<number>")) {
+	cartdata.setCartNumber(GetXmlValue("number",f0[i]).toUInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<groupName>")) {
+	cartdata.setCategory(GetXmlValue("groupName",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<title>")) {
+	cartdata.setTitle(GetXmlValue("title",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<artist>")) {
+	cartdata.setArtist(GetXmlValue("artist",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<album>")) {
+	cartdata.setAlbum(GetXmlValue("album",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<label>")) {
+	cartdata.setLabel(GetXmlValue("label",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<client>")) {
+	cartdata.setClient(GetXmlValue("client",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<agency>")) {
+	cartdata.setAgency(GetXmlValue("agency",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<composer>")) {
+	cartdata.setComposer(GetXmlValue("composer",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<publisher>")) {
+	cartdata.setPublisher(GetXmlValue("publisher",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<conductor>")) {
+	cartdata.setConductor(GetXmlValue("conductor",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<userDefined>")) {
+	cartdata.setUserDefined(GetXmlValue("userDefined",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<year>")) {
+	cartdata.setReleaseYear(GetXmlValue("year",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<forcedLength>")) {
+	cartdata.
+	  setForcedLength(RDSetTimeLength(GetXmlValue("forcedLength",f0[i]).
+					  toString()));
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<averageLength>")) {
+	cartdata.
+	  setAverageLength(RDSetTimeLength(GetXmlValue("averageLength",f0[i]).
+					   toString()));
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<lengthDeviation>")) {
+	cartdata.setLengthDeviation(RDSetTimeLength(GetXmlValue("lengthDeviation",f0[i]).
+					   toString()));
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<averageHookLength>")) {
+	cartdata.
+	  setAverageHookLength(RDSetTimeLength(GetXmlValue("averageHookLength",f0[i]).
+					   toString()));
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<averageSegueLength>")) {
+	cartdata.
+	  setAverageSegueLength(RDSetTimeLength(GetXmlValue("averageSegueLength",f0[i]).
+					   toString()));
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<cutQuantity>")) {
+	cartdata.setCutQuantity(GetXmlValue("cutQuantity",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<lastCutPlayed>")) {
+	cartdata.setLastCutPlayed(GetXmlValue("lastCutPlayed",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<enforceLength>")) {
+	cartdata.setEnforceLength(GetXmlValue("enforceLength",f0[i]).toBool());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<asyncronous>")) {
+	cartdata.setAsyncronous(GetXmlValue("asyncronous",f0[i]).toBool());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<owner>")) {
+	cartdata.setOwner(GetXmlValue("owner",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<metadataDatetime>")) {
+	cartdata.setMetadataDatetime(GetXmlValue("metadataDatetime",f0[i]).
+				     toDateTime());
+	cartdata.setMetadataFound(true);
+      }
+
+      if(f0[i]=="</cart>") {
+	istate=0;
+      }
+
+      if(f0[i]=="<cutList>") {
+	data->push_back(RDWaveData());
+	data->back()=cartdata;
+	istate=2;
+      }
+      break;
+
+    case 2:  // Cut List
+      if(f0[i]=="<cut>") {  // Start of new cut
+	data->push_back(RDWaveData());
+	data->back()=cartdata;
+	settings=new RDSettings();
+	istate=3;
+      }
+      if(f0[i]=="</cutList>") { // End of cut
+	istate=1;
+      }
+      break;
+
+    case 3:  // Cut Object
+      if(f0[i].contains("<description>")) {
+	data->
+	  back().setDescription(GetXmlValue("description",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<cutNumber>")) {
+	data->back().setCutNumber(GetXmlValue("cutNumber",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<cutName>")) {
+	data->back().setCutName(GetXmlValue("cutName",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<evergreen>")) {
+	data->back().setEvergreen(GetXmlValue("evergreen",f0[i]).toBool());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<outcue>")) {
+	data->back().setOutCue(GetXmlValue("outcue",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<isrc>")) {
+	data->back().setIsrc(GetXmlValue("isrc",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<isci>")) {
+	data->back().setIsci(GetXmlValue("isci",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<length>")) {
+	data->back().setLength(GetXmlValue("length",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<originName>")) {
+	data->back().setOriginator(GetXmlValue("originName",f0[i]).toString());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<originDatetime>")) {
+	data->back().setOriginationDate(GetXmlValue("originDatetime",f0[i]).
+				    toDateTime().date());
+	data->back().setOriginationTime(GetXmlValue("originDatetime",f0[i]).
+				    toDateTime().time());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<playCounter>")) {
+	data->back().setPlayCounter(GetXmlValue("playCounter",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<startDatetime>")) {
+	data->back().setStartDate(GetXmlValue("startDatetime",f0[i]).
+				    toDateTime().date());
+	data->back().setStartTime(GetXmlValue("startDatetime",f0[i]).
+				    toDateTime().time());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<endDatetime>")) {
+	data->back().setEndDate(GetXmlValue("endDatetime",f0[i]).
+			    toDateTime().date());
+	data->back().setEndTime(GetXmlValue("endDatetime",f0[i]).
+			    toDateTime().time());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<lastPlayDatetime>")) {
+	data->back().setLastPlayDatetime(GetXmlValue("lastPlayDatetime",f0[i]).
+			    toDateTime());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<codingFormat>")) {
+	settings->setFormat((RDSettings::Format)GetXmlValue("codingFormat",f0[i]).
+		     toUInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<sampleRate>")) {
+	settings->setSampleRate(GetXmlValue("sampleRate",f0[i]).toUInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<bitRate>")) {
+	settings->setBitRate(GetXmlValue("bitRate",f0[i]).toUInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<channels>")) {
+	settings->setChannels(GetXmlValue("channels",f0[i]).toUInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<playGain>")) {
+	data->back().setPlayGain(GetXmlValue("playGain",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<startPoint>")) {
+	data->back().setStartPos(GetXmlValue("startPoint",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<endPoint>")) {
+	data->back().setEndPos(GetXmlValue("endPoint",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<segueStartPoint>")) {
+	data->back().setSegueStartPos(GetXmlValue("segueStartPoint",f0[i]).
+				      toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<segueEndPoint>")) {
+	data->back().setSegueEndPos(GetXmlValue("segueEndPoint",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<segueGain>")) {
+	data->back().setSegueGain(GetXmlValue("segueGain",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<talkStartPoint>")) {
+	data->back().setIntroStartPos(GetXmlValue("talkStartPoint",f0[i]).
+				      toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<talkEndPoint>")) {
+	data->back().setIntroEndPos(GetXmlValue("talkEndPoint",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<hookStartPoint>")) {
+	data->back().setHookStartPos(GetXmlValue("hookStartPoint",f0[i]).
+				      toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<hookEndPoint>")) {
+	data->back().setHookEndPos(GetXmlValue("hookEndPoint",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<fadeupPoint>")) {
+	data->back().setFadeUpPos(GetXmlValue("fadeupPoint",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      if(f0[i].contains("<fadedownPoint>")) {
+	data->back().setFadeDownPos(GetXmlValue("fadedownPoint",f0[i]).toInt());
+	cartdata.setMetadataFound(true);
+      }
+      
+      if(f0[i]=="</cut>") { // End of cut
+	data->back().setAudioSettings(*settings);
+	delete settings;
+	settings=NULL;
+	istate=2;
+      }
+      break;
+    }
+  }
+  return data->size();
+}
+
+
+QVariant RDCart::GetXmlValue(const QString &tag,const QString &line)
+{
+  bool ok=false;
+  QString value=line;
+  value=value.remove("<"+tag+">").remove("</"+tag+">");
+
+  value.toUInt(&ok);
+  if(ok) {
+    return QVariant(value.toUInt());
+  }
+  value.toInt(&ok);
+  if(ok) {
+    return QVariant(value.toInt());
+  }
+
+  return QVariant(RDXmlUnescape(value));
+}
+
+
 QString RDCart::GetNextCut(RDSqlQuery *q) const
 {
   QString cutname;
@@ -1732,9 +2119,9 @@ void RDCart::SetRow(const QString &param,const QDateTime &value) const
   RDSqlQuery *q;
   QString sql;
 
-  sql=QString().sprintf("UPDATE CART SET %s=\"%s\" WHERE NUMBER=%u",
+  sql=QString().sprintf("UPDATE CART SET %s=%s WHERE NUMBER=%u",
 			(const char *)param,
-			(const char *)value.toString("yyyy-MM-dd hh:mm:ss"),
+			(const char *)RDCheckDateTime(value,"yyyy-MM-dd hh:mm:ss"),
 			cart_number);
   q=new RDSqlQuery(sql);
   delete q;
@@ -1746,9 +2133,9 @@ void RDCart::SetRow(const QString &param,const QDate &value) const
   RDSqlQuery *q;
   QString sql;
 
-  sql=QString().sprintf("UPDATE CART SET %s=\"%s\" WHERE NUMBER=%u",
+  sql=QString().sprintf("UPDATE CART SET %s=%s WHERE NUMBER=%u",
 			(const char *)param,
-			(const char *)value.toString("yyyy-MM-dd"),
+			(const char *)RDCheckDateTime(value,"yyyy-MM-dd"),
 			cart_number);
   q=new RDSqlQuery(sql);
   delete q;
