@@ -2,9 +2,7 @@
 //
 // Get the status of the Rivendell system daemons.
 //
-//   (C) Copyright 2002-2003 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: rdcheck_daemons.h,v 1.7 2010/07/29 19:32:33 cvs Exp $
+//   (C) Copyright 2002-2003,2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -25,7 +23,6 @@
 
 #include <qstring.h>
 
-
 void RDInitializeDaemons();
 bool RDCheckDaemon(QString name);
 bool RDCheckDaemons();
@@ -33,5 +30,5 @@ bool RDStartDaemons();
 bool RDKillDaemons();
 
 
-#endif
+#endif  // RDCHECK_DAEMONS_H
 
