@@ -2,9 +2,7 @@
 //
 // Abstract a Rivendell Cut.
 //
-//   (C) Copyright 2002-2004 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: rdcut.cpp,v 1.76.6.10.2.2 2014/07/15 20:02:22 cvs Exp $
+//   (C) Copyright 2002-2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -829,6 +827,7 @@ bool RDCut::copyTo(RDStation *station,RDUser *user,
 #else
   QString sql;
   RDSqlQuery *q;
+  RDSqlQuery *q1;
   bool ret=true;
 
   //
@@ -891,6 +890,22 @@ bool RDCut::copyTo(RDStation *station,RDUser *user,
   q=new RDSqlQuery(sql);
   delete q;
 
+  //
+  // Copy the Cut Events
+  //
+  sql=QString("select NUMBER,POINT from CUT_EVENTS ")+
+    "where CUT_NAME=\""+cutName()+"\"";
+  q=new RDSqlQuery(sql);
+  while(q->next()) {
+    sql=QString("insert into CUT_EVENTS set ")+
+      "CUT_NAME=\""+cutname+"\","+
+      QString().sprintf("NUMBER=%d,",q->value(0).toInt())+
+      QString().sprintf("POINT=%d",q->value(1).toInt());
+    q1=new RDSqlQuery(sql);
+    delete q1;
+  }
+  delete q;
+
   //
   // Copy the Audio
   //
@@ -912,38 +927,54 @@ void RDCut::getMetadata(RDWaveData *data) const
   QString sql;
   RDSqlQuery *q;
 
-  sql=QString().sprintf("select DESCRIPTION,OUTCUE,ISRC,ISCI,ORIGIN_DATETIME,\
-                         START_DATETIME,END_DATETIME,START_DAYPART,END_DAYPART,\
-                         SEGUE_START_POINT,\
-                         SEGUE_END_POINT,TALK_START_POINT,TALK_END_POINT,\
-                         START_POINT,END_POINT,HOOK_START_POINT,\
-                         HOOK_END_POINT,FADEUP_POINT,FADEDOWN_POINT \
-                         from CUTS where CUT_NAME=\"%s\"",
-			(const char *)cut_name);
+  sql=QString("select ")+  
+    "CUT_NAME,"+           // 00
+    "DESCRIPTION,"+        // 01
+    "OUTCUE,"+             // 02
+    "ISRC,"+               // 03
+    "ISCI,"+               // 04
+    "ORIGIN_DATETIME,"+    // 05
+    "START_DATETIME,"+     // 06
+    "END_DATETIME,"+       // 07
+    "START_DAYPART,"+      // 08
+    "END_DAYPART,"+        // 09
+    "SEGUE_START_POINT,"+  // 10
+    "SEGUE_END_POINT,"+    // 11
+    "TALK_START_POINT,"+   // 12
+    "TALK_END_POINT,"+     // 13
+    "START_POINT,"+        // 14
+    "END_POINT,"+          // 15
+    "HOOK_START_POINT,"+   // 16
+    "HOOK_END_POINT,"+     // 17
+    "FADEUP_POINT,"+       // 18
+    "FADEDOWN_POINT "+     // 19
+    "from CUTS where CUT_NAME=\""+cut_name+"\"";
   q=new RDSqlQuery(sql);
   if(q->first()) {
-    data->setDescription(q->value(0).toString());
-    data->setOutCue(q->value(1).toString());
-    data->setIsrc(q->value(2).toString());
-    data->setIsci(q->value(3).toString());
-    data->setOriginationDate(q->value(4).toDate());
-    data->setOriginationTime(q->value(4).toTime());
-    data->setStartDate(q->value(5).toDate());
-    data->setStartTime(q->value(5).toTime());
-    data->setEndDate(q->value(6).toDate());
-    data->setEndTime(q->value(6).toTime());
-    data->setDaypartStartTime(q->value(7).toTime());
-    data->setDaypartEndTime(q->value(8).toTime());
-    data->setSegueStartPos(q->value(9).toInt());
-    data->setSegueEndPos(q->value(10).toInt());
-    data->setIntroStartPos(q->value(11).toInt());
-    data->setIntroEndPos(q->value(12).toInt());
-    data->setStartPos(q->value(13).toInt());
-    data->setEndPos(q->value(14).toInt());
-    data->setHookStartPos(q->value(15).toInt());
-    data->setHookEndPos(q->value(16).toInt());
-    data->setFadeUpPos(q->value(17).toInt());
-    data->setFadeDownPos(q->value(18).toInt());
+    data->setCutName(q->value(0).toString());
+    data->setCutNumber(RDCut::cutNumber(q->value(0).toString()));
+    data->setDescription(q->value(1).toString());
+    data->setOutCue(q->value(2).toString());
+    data->setIsrc(q->value(3).toString());
+    data->setIsci(q->value(4).toString());
+    data->setOriginationDate(q->value(5).toDate());
+    data->setOriginationTime(q->value(5).toTime());
+    data->setStartDate(q->value(6).toDate());
+    data->setStartTime(q->value(6).toTime());
+    data->setEndDate(q->value(7).toDate());
+    data->setEndTime(q->value(7).toTime());
+    data->setDaypartStartTime(q->value(8).toTime());
+    data->setDaypartEndTime(q->value(9).toTime());
+    data->setSegueStartPos(q->value(10).toInt());
+    data->setSegueEndPos(q->value(11).toInt());
+    data->setIntroStartPos(q->value(12).toInt());
+    data->setIntroEndPos(q->value(13).toInt());
+    data->setStartPos(q->value(14).toInt());
+    data->setEndPos(q->value(15).toInt());
+    data->setHookStartPos(q->value(16).toInt());
+    data->setHookEndPos(q->value(17).toInt());
+    data->setFadeUpPos(q->value(18).toInt());
+    data->setFadeDownPos(q->value(19).toInt());
     data->setMetadataFound(true);
   }
   delete q;
@@ -1045,8 +1076,9 @@ void RDCut::setMetadata(RDWaveData *data) const
   }
   if(data->daypartStartTime().isValid()&&data->daypartEndTime().isValid()&&
      (data->daypartStartTime()<data->daypartEndTime())) {
-    sql+="START_DAYPART=\""+data->daypartStartTime().toString("hh:mm:ss")+"\","+
-      "END_DAYPART=\""+data->daypartEndTime().toString("hh:mm:ss")+"\",";
+    sql+="START_DAYPART="+
+      RDCheckDateTime(data->daypartStartTime(),"hh:mm:ss")+","+
+      "END_DAYPART="+RDCheckDateTime(data->daypartEndTime(),"hh:mm:ss")+",";
   }
   if((data->hookStartPos()>=data->startPos())&&
      (data->hookStartPos()<=data->endPos())&&
@@ -1064,32 +1096,36 @@ void RDCut::setMetadata(RDWaveData *data) const
      (data->fadeDownPos()<=data->endPos())) {
     sql+=QString().sprintf("FADEDOWN_POINT=%d,",data->fadeDownPos());
   }
-  if((data->startDate()>QDate(1900,1,1))&&(data->endDate().year()<8000)) {
-    if(data->startTime().isValid()) {
-    sql+=QString().sprintf("START_DATETIME=\"%s %s\",",
-			   (const char *)data->startDate().
-			   toString("yyyy-MM-dd"),
-			   (const char *)data->startTime().
-			   toString("hh:mm:ss"));
-    }
-    else {
-      sql+=QString().sprintf("START_DATETIME=\"%s 00:00:00\",",
-			     (const char *)data->startDate().
-			     toString("yyyy-MM-dd"));
-    }
+  if(data->startDate().isValid() && 
+     (data->startDate()>QDate(1900,1,1))&&(data->endDate().year()<8000)) {
+    
+    /* Reworked, 
+     *   if date not valid, do nothing
+     *   if time valid use date + time
+     *   else use date + 00:00:00
+     */
+    QDateTime startDateTime(data->startDate());
+      
+    if(data->startTime().isValid())
+      startDateTime.setTime(data->startTime());
+    else
+      startDateTime.setTime(QTime(0,0,0));
+      
+    sql+=QString().sprintf("START_DATETIME=%s,",
+        (const char *)RDCheckDateTime(startDateTime,"yyyy-MM-dd hh:mm:ss"));
+      
     if(data->endDate().isValid()&&(data->endDate().year()<8000)) {
-      if(data->endTime().isValid()) {
-	sql+=QString().sprintf("END_DATETIME=\"%s %s\",",
-			       (const char *)data->endDate().
-			       toString("yyyy-MM-dd"),
-			       (const char *)data->endTime().
-			       toString("hh:mm:ss"));
-      }
-      else {
-	sql+=QString().sprintf("END_DATETIME=\"%s 23:59:59\",",
-			       (const char *)data->endDate().
-			       toString("yyyy-MM-dd"));
-      }
+      
+      // Reworked as START_DATETIME
+      QDateTime endDateTime(data->endDate());
+      
+      if(data->endTime().isValid())
+        endDateTime.setTime(data->endTime());
+      else
+        endDateTime.setTime(QTime(23,59,59));
+      
+      sql+=QString().sprintf("END_DATETIME=%s,",
+            (const char *)RDCheckDateTime(endDateTime,"yyyy-MM-dd hh:mm:ss"));
     }
   }
   if(sql.right(1)==",") {
@@ -1099,10 +1135,28 @@ void RDCut::setMetadata(RDWaveData *data) const
     sprintf(" where CUT_NAME=\"%s\"",(const char *)cut_name.utf8());
   RDSqlQuery *q=new RDSqlQuery(sql);
   delete q;
+
+  //
+  // Sanity Check: NEVER permit the 'description' field to be empty.
+  //
+  sql=QString("select DESCRIPTION from CUTS where ")+
+    "CUT_NAME=\""+RDEscapeString(cut_name)+"\"";
+  q=new RDSqlQuery(sql);
+  if(q->first()) {
+    if(q->value(0).toString().isEmpty()) {
+      sql=QString("update CUTS set ")+
+	QString().sprintf("DESCRIPTION=\"Cut %03d\"",
+			  RDCut::cutNumber(cut_name))+
+	" where CUT_NAME=\""+RDEscapeString(cut_name)+"\"";
+      delete q;
+      q=new RDSqlQuery(sql);
+    }
+  }
+  delete q;
 }
 
 
-QString RDCut::xml() const
+QString RDCut::xml(RDSettings *settings) const
 {
 #ifdef WIN32
   return QString();
@@ -1111,18 +1165,48 @@ QString RDCut::xml() const
   RDSqlQuery *q;
   QString ret="";
 
-  sql=QString().sprintf("select EVERGREEN,DESCRIPTION,OUTCUE,ISRC,ISCI,\
-                         LENGTH,ORIGIN_DATETIME,START_DATETIME,END_DATETIME,\
-                         SUN,MON,TUE,WED,THU,FRI,SAT,START_DAYPART,END_DAYPART,\
-                         ORIGIN_NAME,WEIGHT,LAST_PLAY_DATETIME,\
-                         PLAY_COUNTER,LOCAL_COUNTER,VALIDITY,CODING_FORMAT,\
-                         SAMPLE_RATE,BIT_RATE,CHANNELS,PLAY_GAIN,START_POINT,\
-                         END_POINT,FADEUP_POINT,FADEDOWN_POINT,\
-                         SEGUE_START_POINT,SEGUE_END_POINT,SEGUE_GAIN\
-                         HOOK_START_POINT,HOOK_END_POINT,\
-                         TALK_START_POINT,TALK_END_POINT \
-                         from CUTS where CUT_NAME=\"%s\"",
-			(const char *)cut_name);
+  sql=QString("select ")+
+    "EVERGREEN,"+              // 00
+    "DESCRIPTION,"+            // 01
+    "OUTCUE,"+                 // 02
+    "ISRC,"+                   // 03
+    "ISCI,"+                   // 04
+    "LENGTH,"+                 // 05
+    "ORIGIN_DATETIME,"+        // 06
+    "START_DATETIME,"+         // 07
+    "END_DATETIME,"+           // 08
+    "SUN,"+                    // 09
+    "MON,"+                    // 10
+    "TUE,"+                    // 11
+    "WED,"+                    // 12
+    "THU,"+                    // 13
+    "FRI,"+                    // 14
+    "SAT,"+                    // 15
+    "START_DAYPART,"+          // 16
+    "END_DAYPART,"+            // 17
+    "ORIGIN_NAME,"+            // 18
+    "WEIGHT,"+                 // 19
+    "LAST_PLAY_DATETIME,"+     // 20
+    "PLAY_COUNTER,"+           // 21
+    "LOCAL_COUNTER,"+          // 22
+    "VALIDITY,"+               // 23
+    "CODING_FORMAT,"+          // 24
+    "SAMPLE_RATE,"+            // 25
+    "BIT_RATE,"+               // 26
+    "CHANNELS,"+               // 27
+    "PLAY_GAIN,"+              // 28
+    "START_POINT,"+            // 29
+    "END_POINT,"+              // 30
+    "FADEUP_POINT,"+           // 31
+    "FADEDOWN_POINT,"+         // 32
+    "SEGUE_START_POINT,"+      // 33
+    "SEGUE_END_POINT,"+        // 34
+    "SEGUE_GAIN,"+             // 35
+    "HOOK_START_POINT,"+       // 36
+    "HOOK_END_POINT,"+         // 37
+    "TALK_START_POINT,"+       // 38
+    "TALK_END_POINT "+         // 39
+    "from CUTS where CUT_NAME=\""+cut_name+"\"";
   q=new RDSqlQuery(sql);
   if(q->first()) {
     ret+="<cut>\n";
@@ -1176,12 +1260,18 @@ QString RDCut::xml() const
     ret+="  "+RDXmlField("weight",q->value(19).toUInt());
     ret+="  "+RDXmlField("lastPlayDatetime",q->value(20).toDateTime());
     ret+="  "+RDXmlField("playCounter",q->value(21).toUInt());
-    ret+="  "+RDXmlField("localCounter",q->value(22).toUInt());
-    ret+="  "+RDXmlField("validity",q->value(23).toUInt());
-    ret+="  "+RDXmlField("codingFormat",q->value(24).toUInt());
-    ret+="  "+RDXmlField("sampleRate",q->value(25).toUInt());
-    ret+="  "+RDXmlField("bitRate",q->value(26).toUInt());
-    ret+="  "+RDXmlField("channels",q->value(27).toUInt());
+    if(settings==NULL) {
+      ret+="  "+RDXmlField("codingFormat",q->value(24).toUInt());
+      ret+="  "+RDXmlField("sampleRate",q->value(25).toUInt());
+      ret+="  "+RDXmlField("bitRate",q->value(26).toUInt());
+      ret+="  "+RDXmlField("channels",q->value(27).toUInt());
+    }
+    else {
+      ret+="  "+RDXmlField("codingFormat",(int)settings->format());
+      ret+="  "+RDXmlField("sampleRate",settings->sampleRate());
+      ret+="  "+RDXmlField("bitRate",settings->bitRate());
+      ret+="  "+RDXmlField("channels",settings->channels());
+    }
     ret+="  "+RDXmlField("playGain",q->value(28).toUInt());
     ret+="  "+RDXmlField("startPoint",q->value(29).toInt());
     ret+="  "+RDXmlField("endPoint",q->value(30).toInt());
@@ -1588,9 +1678,9 @@ void RDCut::SetRow(const QString &param,const QDateTime &value) const
   RDSqlQuery *q;
   QString sql;
 
-  sql=QString().sprintf("UPDATE CUTS SET %s=\"%s\" WHERE CUT_NAME=\"%s\"",
+  sql=QString().sprintf("UPDATE CUTS SET %s=%s WHERE CUT_NAME=\"%s\"",
 			(const char *)param,
-			(const char *)value.toString("yyyy-MM-dd hh:mm:ss"),
+			(const char *)RDCheckDateTime(value,"yyyy-MM-dd hh:mm:ss"),
 			(const char *)cut_name);
   q=new RDSqlQuery(sql,cut_db);
   delete q;
@@ -1602,9 +1692,9 @@ void RDCut::SetRow(const QString &param,const QDate &value) const
   RDSqlQuery *q;
   QString sql;
 
-  sql=QString().sprintf("UPDATE CUTS SET %s=\"%s\" WHERE CUT_NAME=\"%s\"",
+  sql=QString().sprintf("UPDATE CUTS SET %s=%s WHERE CUT_NAME=\"%s\"",
 			(const char *)param,
-			(const char *)value.toString("yyyy-MM-dd"),
+			(const char *)RDCheckDateTime(value,"yyyy-MM-dd"),
 			(const char *)cut_name);
   q=new RDSqlQuery(sql,cut_db);
   delete q;
@@ -1615,9 +1705,9 @@ void RDCut::SetRow(const QString &param,const QTime &value) const
 {
   RDSqlQuery *q;
   QString sql;
-  sql=QString().sprintf("UPDATE CUTS SET %s=\"%s\" WHERE CUT_NAME=\"%s\"",
+  sql=QString().sprintf("UPDATE CUTS SET %s=%s WHERE CUT_NAME=\"%s\"",
 			(const char *)param,
-			(const char *)value.toString("hh:mm:ss"),
+			(const char *)RDCheckDateTime(value,"hh:mm:ss"),
 			(const char *)cut_name);
   q=new RDSqlQuery(sql,cut_db);
   delete q;
