@@ -2,9 +2,7 @@
 //
 // A Qt-based application for testing General Purpose Outputs (GPO).
 //
-//   (C) Copyright 2002-2003 Fred Gleason <fredg@paravelsystems.com>
-//
-//    $Id: rddatepicker.cpp,v 1.3.8.1 2013/01/22 18:33:38 cvs Exp $
+//   (C) Copyright 2002-2003,2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU Library General Public License 
@@ -31,9 +29,8 @@
 //
 // Global Classes
 //
-RDDatePicker::RDDatePicker(int low_year,int high_year,
-			 QWidget *parent,const char *name)
-  :QWidget(parent,name)
+RDDatePicker::RDDatePicker(int low_year,int high_year,QWidget *parent)
+  :QWidget(parent)
 {
   pick_low_year=low_year;
   pick_high_year=high_year;
