@@ -2,9 +2,7 @@
 //
 // Abstract Rivendell Log Events
 //
-//   (C) Copyright 2002-2003 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: rdlog_event.h,v 1.36.6.4.2.1 2014/05/22 19:37:44 cvs Exp $
+//   (C) Copyright 2002-2003,2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -27,7 +25,6 @@
 
 #include <rdlog_line.h>
 
-
 #ifndef RDLOG_EVENT_H
 #define RDLOG_EVENT_H
 
@@ -45,6 +42,7 @@ class RDLogEvent
    void setLogName(QString logname);
    QString serviceName() const;
    int load(bool track_ptrs=false);
+   void saveModified(bool update_tracks=true);
    void save(bool update_tracks=true,int line=-1);
    int append(const QString &logname,bool track_ptrs=false);
    int validate(QString *report,const QDate &date);
@@ -75,6 +73,8 @@ class RDLogEvent
   private:
    int LoadLines(const QString &log_table,int id_offset,bool track_ptrs);
    void SaveLine(int line);
+   void InsertLines(QString values);
+   void InsertLineValues(QString *query, int line);
    void LoadNowNext(unsigned from_line);
    QString log_name;
    QString log_service_name;
