@@ -2,9 +2,7 @@
 //
 // List Rivendell Workstations
 //
-//   (C) Copyright 2002-2003 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: list_stations.cpp,v 1.25 2008/09/18 19:02:13 fredg Exp $
+//   (C) Copyright 2002-2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -38,9 +36,8 @@
 #include <edit_station.h>
 #include <add_station.h>
 
-
-ListStations::ListStations(QWidget *parent,const char *name)
-  : QDialog(parent,name,true)
+ListStations::ListStations(QWidget *parent)
+  : QDialog(parent,"",true)
 {
   //
   // Fix the Window Size
@@ -63,7 +60,7 @@ ListStations::ListStations(QWidget *parent,const char *name)
   //
   //  Add Button
   //
-  QPushButton *add_button=new QPushButton(this,"add_button");
+  QPushButton *add_button=new QPushButton(this);
   add_button->setGeometry(410,30,80,50);
   add_button->setFont(font);
   add_button->setText(tr("&Add"));
@@ -72,7 +69,7 @@ ListStations::ListStations(QWidget *parent,const char *name)
   //
   //  Edit Button
   //
-  QPushButton *edit_button=new QPushButton(this,"edit_button");
+  QPushButton *edit_button=new QPushButton(this);
   edit_button->setGeometry(410,90,80,50);
   edit_button->setFont(font);
   edit_button->setText(tr("&Edit"));
@@ -81,7 +78,7 @@ ListStations::ListStations(QWidget *parent,const char *name)
   //
   //  Delete Button
   //
-  QPushButton *delete_button=new QPushButton(this,"delete_button");
+  QPushButton *delete_button=new QPushButton(this);
   delete_button->setGeometry(410,150,80,50);
   delete_button->setFont(font);
   delete_button->setText(tr("&Delete"));
@@ -90,7 +87,7 @@ ListStations::ListStations(QWidget *parent,const char *name)
   //
   //  Close Button
   //
-  QPushButton *close_button=new QPushButton(this,"close_button");
+  QPushButton *close_button=new QPushButton(this);
   close_button->setGeometry(410,240,80,50);
   close_button->setDefault(true);
   close_button->setFont(font);
@@ -100,10 +97,9 @@ ListStations::ListStations(QWidget *parent,const char *name)
   //
   // Station List Box
   //
-  list_box=new QListBox(this,"list_box");
+  list_box=new QListBox(this);
   list_box->setGeometry(10,30,390,260);
-  QLabel *list_box_label=new QLabel(list_box,tr("Ho&sts:"),
-				    this,"list_box_label");
+  QLabel *list_box_label=new QLabel(list_box,tr("Ho&sts:"),this);
   list_box_label->setFont(font);
   list_box_label->setGeometry(14,10,85,19);
   connect(list_box,SIGNAL(doubleClicked(QListBoxItem *)),
@@ -135,7 +131,7 @@ void ListStations::addData()
 {
   QString stationname;
 
-  AddStation *add_station=new AddStation(&stationname,this,"add_station");
+  AddStation *add_station=new AddStation(&stationname,this);
   if(add_station->exec()<0) {
     DeleteStation(stationname);
     delete add_station;
@@ -151,8 +147,7 @@ void ListStations::editData()
   if(list_box->currentItem()<0) {
     return;
   }
-  EditStation *edit_station=new EditStation(list_box->currentText(),
-					    this,"edit_station");
+  EditStation *edit_station=new EditStation(list_box->currentText(),this);
   edit_station->exec();
   delete edit_station;
 }
@@ -214,84 +209,96 @@ void ListStations::DeleteStation(QString name)
   RDSqlQuery *q1;
 
   sql=QString().sprintf("delete from DECKS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
+  q=new RDSqlQuery(sql);
+  delete q;
+  sql=QString("delete from DECK_EVENTS where ")+
+    "STATION_NAME=\""+RDEscapeString(name)+"\"";
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from TTYS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from AUDIO_PORTS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from RECORDINGS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=
     QString().sprintf("delete from SERVICE_PERMS where STATION_NAME=\"%s\"",
-		      (const char *)name);
+		      (const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from RDAIRPLAY where STATION=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
+  q=new RDSqlQuery(sql);
+  delete q;
+  sql=QString().sprintf("delete from RDAIRPLAY_CHANNELS where STATION_NAME=\"%s\"",
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from RDPANEL where STATION=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
+  q=new RDSqlQuery(sql);
+  delete q;
+  sql=QString().sprintf("delete from RDPANEL_CHANNELS where STATION_NAME=\"%s\"",
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from RDLOGEDIT where STATION=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from MATRICES where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from INPUTS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from OUTPUTS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from VGUEST_RESOURCES where\
                            STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from RDLIBRARY where STATION=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from GPIS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from HOSTVARS where STATION_NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from STATIONS where NAME=\"%s\"",
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=
     QString().sprintf("delete from REPORT_STATIONS where STATION_NAME=\"%s\"",
-		      (const char *)name);
+		      (const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().sprintf("delete from PANELS where (TYPE=%d && OWNER=\"%s\")",
 			RDAirPlayConf::StationPanel,
-			(const char *)name);
+			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
   sql=QString().
     sprintf("delete from EXTENDED_PANELS where (TYPE=%d && OWNER=\"%s\")",
-	    RDAirPlayConf::StationPanel,(const char *)name);
+	    RDAirPlayConf::StationPanel,(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
 
@@ -317,5 +324,9 @@ void ListStations::DeleteStation(QString name)
 			(const char *)RDEscapeString(name));
   q=new RDSqlQuery(sql);
   delete q;
+  sql=QString().sprintf("delete from RDHOTKEYS where STATION_NAME=\"%s\"",
+			(const char *)RDEscapeString(name));
+  q=new RDSqlQuery(sql);
+  delete q;
 }
 
