@@ -2,9 +2,7 @@
 //
 // Add a Rivendell Log Manager Event
 //
-//   (C) Copyright 2002-2004 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: add_event.h,v 1.5.8.1 2012/04/23 17:22:47 cvs Exp $
+//   (C) Copyright 2002-2004,2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -33,12 +31,11 @@
 
 #include <rdevent.h>
 
-
 class AddEvent : public QDialog
 {
   Q_OBJECT
  public:
-  AddEvent(QString *logname,QWidget *parent=0,const char *name=0);
+  AddEvent(QString *logname,QWidget *parent=0);
   ~AddEvent();
   QSize sizeHint() const;
   QSizePolicy sizePolicy() const;
