@@ -2,15 +2,12 @@
 //
 //   The Import Carts ListView widget for RDLogManager.
 //
-//   (C) Copyright 2002-2004 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: import_listview.h,v 1.11 2008/04/02 16:47:28 fredg Exp $
+//   (C) Copyright 2002-2004,2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
 //   published by the Free Software Foundation.
 //
-//
 //   This program is distributed in the hope that it will be useful,
 //   but WITHOUT ANY WARRANTY; without even the implied warranty of
 //   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,7 +17,6 @@
 //   License along with this program; if not, write to the Free Software
 //   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
-//
 
 #ifndef IMPORT_LISTVIEW_H
 #define IMPORT_LISTVIEW_H
@@ -36,7 +32,7 @@ class ImportListView : public QListView
   Q_OBJECT
 
  public:
-  ImportListView(QWidget *parent,const char *name=0);
+  ImportListView(QWidget *parent);
   void setForceTrans(RDLogLine::TransType trans);
   void setAllowStop(bool state);
   void setAllowFirstTrans(bool state);
@@ -46,6 +42,7 @@ class ImportListView : public QListView
 
  signals:
   void sizeChanged(int size);
+  void lengthChanged(int msecs);
 
  private slots:
   void aboutToShowData();
