@@ -2,9 +2,7 @@
 //
 // Load Switcher drivers for ripcd(8)
 //
-//   (C) Copyright 2002-2004 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: loaddrivers.cpp,v 1.1.8.9 2014/02/17 02:19:03 cvs Exp $
+//   (C) Copyright 2002-2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -32,12 +30,14 @@
 #include <bt16x2.h>
 #include <bt8x2.h>
 #include <btacs82.h>
+#include <btadms4422.h>
 #include <btgpi16.h>
 #include <btsentinel4web.h>
 #include <btsrc16.h>
 #include <btsrc8iii.h>
 #include <btss124.h>
 #include <btss164.h>
+#include <btss41mlr.h>
 #include <btss42.h>
 #include <btss44.h>
 #include <btss82.h>
@@ -93,6 +93,10 @@ bool MainObject::LoadSwitchDriver(int matrix_num)
     ripcd_switcher[matrix_num]=new BtAcs82(matrix,this);
     break;
 
+  case RDMatrix::BtAdms4422:
+    ripcd_switcher[matrix_num]=new BtAdms4422(matrix,this);
+    break;
+
   case RDMatrix::BtGpi16:
     ripcd_switcher[matrix_num]=new BtGpi16(matrix,this);
     break;
@@ -117,6 +121,10 @@ bool MainObject::LoadSwitchDriver(int matrix_num)
     ripcd_switcher[matrix_num]=new BtSs164(matrix,this);
     break;
 
+  case RDMatrix::BtSs41Mlr:
+    ripcd_switcher[matrix_num]=new BtSs41Mlr(matrix,this);
+    break;
+
   case RDMatrix::BtSs42:
     ripcd_switcher[matrix_num]=new BtSs42(matrix,this);
     break;
