@@ -2,9 +2,7 @@
 //
 // A Utility for running periodic system maintenance.
 //
-//   (C) Copyright 2008 Fred Gleason <fredg@paravelsystems.com>
-//
-//      $Id: rdmaint.cpp,v 1.9.4.4 2013/11/13 23:36:39 cvs Exp $
+//   (C) Copyright 2008-2016 Fred Gleason <fredg@paravelsystems.com>
 //
 //   This program is free software; you can redistribute it and/or modify
 //   it under the terms of the GNU General Public License version 2 as
@@ -44,8 +42,8 @@
 #include <rdcart.h>
 #include <rdlog.h>
 
-MainObject::MainObject(QObject *parent,const char *name)
-  :QObject(parent,name)
+MainObject::MainObject(QObject *parent)
+  :QObject(parent)
 {
   QString sql;
   RDSqlQuery *q;
@@ -206,9 +204,10 @@ void MainObject::PurgeLogs()
   RDSqlQuery *q;
   QDateTime dt=QDateTime(QDate::currentDate(),QTime::currentTime());
 
-  sql=QString().sprintf("select NAME from LOGS where \
-                         (PURGE_DATE!=\"0000-00-00\")&&(PURGE_DATE<\"%s\")",
-			(const char *)dt.date().toString("yyyy-MM-dd"));
+  sql=QString("select NAME from LOGS where ")+
+    "(PURGE_DATE!=\"0000-00-00\")&&"+
+    "(PURGE_DATE is not null)&&"+
+    "(PURGE_DATE<\""+dt.date().toString("yyyy-MM-dd")+"\")";
   q=new RDSqlQuery(sql);
   while(q->next()) {
     maint_config->
@@ -234,10 +233,9 @@ void MainObject::PurgeElr()
   while(q->next()) {
     QString tablename=q->value(0).toString()+"_SRT";
     tablename.replace(" ","_");
-    sql=QString().sprintf("delete from %s where EVENT_DATETIME<\"%s 00:00:00\"",
-			  (const char *)tablename,
-			  (const char *)dt.addDays(-q->value(1).toInt()).
-			  toString("yyyy-MM-dd"));
+    sql=QString("delete from `")+tablename+"` where "+
+      "EVENT_DATETIME<\""+
+      dt.addDays(-q->value(1).toInt()).toString("yyyy-MM-dd")+" 00:00:00\"";
     q1=new RDSqlQuery(sql);
     delete q1;
   }
@@ -286,6 +284,6 @@ void MainObject::PurgeGpioEvents()
 int main(int argc,char *argv[])
 {
   QApplication a(argc,argv,false);
-  new MainObject(NULL,"main");
+  new MainObject();
   return a.exec();
 }
